@@ -1,6 +1,6 @@
 //===========================================================================
 // @(#) $Name:$
-// @(#) $Id: mcblockc.cc 9691 2017-06-30 09:45:02Z dwm $
+// @(#) $Id: mcblockc.cc 12101 2022-12-04 01:17:28Z dwm $
 //===========================================================================
 //  Copyright (c) Daniel W. McRobb 2017
 //  All rights reserved.
@@ -55,20 +55,17 @@ extern "C" {
 #include <fstream>
 #include <string>
 
-#include "DwmAuth.hh"
-#include "DwmAuthCountedString.hh"
+#include "DwmCredencePeer.hh"
 #include "DwmIO.hh"
 #include "DwmIpv4Routes.hh"
 #include "DwmOptArgs.hh"
 #include "DwmSignal.hh"
-#include "DwmSocket.hh"
 #include "DwmSvnTag.hh"
 #include "DwmSysLogger.hh"
-#include "DwmAuthPeerAuthenticator.hh"
 #include "DwmMcBlockRequestMessage.hh"
 #include "DwmMcBlockResponseMessage.hh"
 
-static const Dwm::SvnTag svntag("@(#) $DwmPath: dwm/mcplex/mcblock/tags/mcblock-0.1.1/apps/mcblockc/mcblockc.cc 9691 $");
+static const Dwm::SvnTag svntag("@(#) $DwmPath: dwm/mcplex/mcblock/tags/mcblock-0.3.3/apps/mcblockc/mcblockc.cc 12101 $");
 
 using namespace std;
 using namespace Dwm;
@@ -95,7 +92,7 @@ static string GetKeyDirectory()
   string  rc;
   string  homeDir = GetHomeDirectory();
   if (! homeDir.empty()) {
-    rc = homeDir + "/.dwmauth";
+    rc = homeDir + "/.credence";
   }
   return rc;
 }
@@ -103,28 +100,13 @@ static string GetKeyDirectory()
 //----------------------------------------------------------------------------
 //!  
 //----------------------------------------------------------------------------
-static bool Authenticate(Dwm::Socket & s, string & theirId,
-                         string & agreedKey)
-{
-  bool  rc = false;
-  Auth::PeerAuthenticator  peerAuth(GetKeyDirectory() + "/id_rsa",
-                                    GetKeyDirectory() + "/known_services");
-  if (peerAuth.Authenticate(s, theirId, agreedKey)) {
-    rc = true;
-  }
-  return rc;
-}
-
-//----------------------------------------------------------------------------
-//!  
-//----------------------------------------------------------------------------
-bool RequestResponse(Socket & s, const string & secret,
+bool RequestResponse(Credence::Peer & peer,
                      const McBlock::RequestMessage & request,
                      McBlock::ResponseMessage & response)
 {
   bool  rc = false;
-  if (request.Write(s, secret)) {
-    if (response.Read(s, secret)) {
+  if (request.Write(peer)) {
+    if (response.Read(peer)) {
       rc = true;
     }
     else {
@@ -140,7 +122,7 @@ bool RequestResponse(Socket & s, const string & secret,
 //----------------------------------------------------------------------------
 //!  
 //----------------------------------------------------------------------------
-bool Connect(const string & host, Socket & s)
+bool Connect(const string & host, Credence::Peer & peer)
 {
   bool  rc = false;
   struct hostent  *hostEntry = gethostbyname(host.c_str());
@@ -148,22 +130,9 @@ bool Connect(const string & host, Socket & s)
     cerr << "Host '" << host << "' not found.\n";
     exit(1);
   }
-  
-  if (s.Open(PF_INET, SOCK_STREAM, 0)) {
-    int  rcvbufSize = 256 * 1024;
-    s.Setsockopt(SOL_SOCKET, SO_RCVBUF, &rcvbufSize, sizeof(rcvbufSize));
-      
-    struct sockaddr_in  servAddr;
-    memset(&servAddr, 0, sizeof(servAddr));
-    servAddr.sin_len = sizeof(servAddr);
-    servAddr.sin_family = AF_INET;
-    servAddr.sin_port = htons(1001);
-    servAddr.sin_addr.s_addr = *(in_addr_t *)hostEntry->h_addr_list[0];
-    if (s.Connect(servAddr)) {
-      int  noDelay = 1;
-      s.Setsockopt(IPPROTO_TCP, TCP_NODELAY, &noDelay, sizeof(noDelay));
-      rc = true;
-    }
+
+  if (peer.Connect(host, 1001)) {
+    rc = true;
   }
   return rc;
 }
@@ -180,23 +149,23 @@ static void Usage(const string & argv0)
        << "       " << argv0 << " [-h host] loghit tableName ipv4addr\n"
        << "       " << argv0 << " [-h host] search ipv4addr\n"
        << "       " << argv0 << " [-h host] get tableName ipv4addr\n"
-       << "       " << argv0 << " [-h host] edit tableName ipv4addr\n";
+       << "       " << argv0 << " [-h host] edit tableName ipv4addr\n"
+       << "       " << argv0 << " [-h host] ping\n";
   return;
 }
 
 //----------------------------------------------------------------------------
 //!  
 //----------------------------------------------------------------------------
-static bool ActivatePrefixes(Socket & s, const string & secret,
-                             const string & table,
+static bool ActivatePrefixes(Credence::Peer & peer,const string & table,
                              const vector<Ipv4Prefix> & prefixes)
 {
   bool  rc = false;
   McBlock::ResponseMessage  resp;
   McBlock::RequestMessage   req(McBlock::ActivateReq(table, prefixes));
-  if (RequestResponse(s, secret, req, resp)) {
+  if (RequestResponse(peer, req, resp)) {
     if (! resp.Json().empty()) {
-      cout << resp.Json().toStyledString() << '\n';
+      cout << resp.Json().dump(2) << '\n';
       rc = true;
     }
   }
@@ -206,16 +175,15 @@ static bool ActivatePrefixes(Socket & s, const string & secret,
 //----------------------------------------------------------------------------
 //!  
 //----------------------------------------------------------------------------
-static bool DeactivatePrefixes(Socket & s, const string & secret,
-                               const string & table,
+static bool DeactivatePrefixes(Credence::Peer & peer, const string & table,
                                const vector<Ipv4Prefix> & prefixes)
 {
   bool  rc = false;
   McBlock::ResponseMessage  resp;
   McBlock::RequestMessage   req(McBlock::DeactivateReq(table, prefixes));
-  if (RequestResponse(s, secret, req, resp)) {
+  if (RequestResponse(peer, req, resp)) {
     if (! resp.Json().empty()) {
-      cout << resp.Json().toStyledString() << '\n';
+      cout << resp.Json().dump(2) << '\n';
       rc = true;
     }
   }
@@ -225,32 +193,30 @@ static bool DeactivatePrefixes(Socket & s, const string & secret,
 //----------------------------------------------------------------------------
 //!  
 //----------------------------------------------------------------------------
-static bool GetActivePrefixes(Socket & s, const string & secret,
-                              const string & table)
+static bool GetActivePrefixes(Credence::Peer & peer, const string & table)
 {
   bool  rc = false;
   McBlock::ResponseMessage  resp;
   McBlock::RequestMessage   req((McBlock::GetActiveReq(table)));
-  if (RequestResponse(s, secret, req, resp)) {
+  if (RequestResponse(peer, req, resp)) {
     if (! resp.Json().empty()) {
-      if (resp.Json().isObject()) {
-        auto  && members = resp.Json().getMemberNames();
-        for (auto tblName : members) {
+      if (resp.Json().is_object()) {
+        for (auto & [key, value] : resp.Json().items()) {
           map<string,Ipv4Routes<uint8_t>>  countryRoutes;
-          if (resp.Json()[tblName].isMember("prefixes")
-              && resp.Json()[tblName]["prefixes"].isArray()) {
-            cout << tblName << ":\n";
-            const Json::Value &  pfxArr = resp.Json()[tblName]["prefixes"];
+          if ((resp.Json()[key].find("prefixes") != resp.Json()[key].end())
+              && resp.Json()[key]["prefixes"].is_array()) {
+            cout << key << ":\n";
+            const nlohmann::json &  pfxArr = resp.Json()[key]["prefixes"];
             for (uint32_t pn = 0; pn < pfxArr.size(); ++pn) {
-              Ipv4Prefix  prefix(pfxArr[pn]["prefix"].asString());
-              string      country(pfxArr[pn]["country"].asString());
+              Ipv4Prefix  prefix(pfxArr[pn]["prefix"].get<string>());
+              string      country(pfxArr[pn]["country"].get<string>());
               cout << setiosflags(ios::left)
                    << "  "
-                   << setw(18) << pfxArr[pn]["prefix"].asString() << ' '
+                   << setw(18) << pfxArr[pn]["prefix"].get<string>() << ' '
                    << setiosflags(ios::right)
-                   << setw(6) << pfxArr[pn]["daysRemaining"].asUInt() << "d "
-                   << setw(3) << pfxArr[pn]["country"].asString() << " ("
-                   << pfxArr[pn]["countryName"].asString() << ")\n"
+                   << setw(6) << pfxArr[pn]["daysRemaining"].get<uint32_t>() << "d "
+                   << setw(3) << pfxArr[pn]["country"].get<string>() << " ("
+                   << pfxArr[pn]["countryName"].get<string>() << ")\n"
                    << resetiosflags(ios::right) << resetiosflags(ios::left);
               countryRoutes[country][prefix] = 1;
             }
@@ -265,7 +231,6 @@ static bool GetActivePrefixes(Socket & s, const string & secret,
                      const pair<string,uint64_t> & a2)
                 { return (a1.second > a2.second); });
           cout << "\n  Addresses covered per country:\n";
-          // std::locale::global(std::locale(""));
           std::cout.imbue(std::locale("en_US.UTF-8"));
           for (auto & cas : countryAddrSpace) {
             cout << "    " << cas.first << ' ' << cas.second << '\n';
@@ -290,28 +255,27 @@ static bool GetActivePrefixes(Socket & s, const string & secret,
 //----------------------------------------------------------------------------
 //!  
 //----------------------------------------------------------------------------
-static bool Search(Socket & s, const string & secret,
-                   const Ipv4Address & addr)
+static bool Search(Credence::Peer & peer, const Ipv4Address & addr)
 {
   bool                      rc = false;
   McBlock::ResponseMessage  resp;
   McBlock::RequestMessage   req((McBlock::SearchReq(addr)));
-  if (RequestResponse(s, secret, req, resp)) {
+  if (RequestResponse(peer, req, resp)) {
     if (! resp.Json().empty()) {
-      if (resp.Json().isObject()) {
-        auto  && members = resp.Json().getMemberNames();
-        for (auto tblName : members) {
-          if (resp.Json()[tblName].isMember("prefix")
-              && resp.Json()[tblName]["prefix"].isString()) {
-            const Json::Value  & match = resp.Json()[tblName];
-            cout << tblName << ":\n"
+      if (resp.Json().is_object()) {
+        // auto  && members = resp.Json().getMemberNames();
+        for (auto & [key, value] : resp.Json().items()) {
+          if ((resp.Json()[key].find("prefix") != resp.Json()[key].end())
+              && resp.Json()[key]["prefix"].is_string()) {
+            const nlohmann::json  & match = resp.Json()[key];
+            cout << key << ":\n"
                  << setiosflags(ios::left)
                  << "  "
-                 << setw(18) << match["prefix"].asString()
+                 << setw(18) << match["prefix"].get<string>()
                  << ' ' << setiosflags(ios::right)
-                 << setw(6) << match["daysRemaining"].asUInt() << "d "
-                 << setw(3) << match["country"].asString() << " ("
-                 << match["countryName"].asString() << ")\n"
+                 << setw(6) << match["daysRemaining"].get<uint32_t>() << "d "
+                 << setw(3) << match["country"].get<string>() << " ("
+                 << match["countryName"].get<string>() << ")\n"
                  << resetiosflags(ios::right) << resetiosflags(ios::left);
           }
           rc = true;
@@ -322,19 +286,42 @@ static bool Search(Socket & s, const string & secret,
   return rc;
 }
 
-            
 //----------------------------------------------------------------------------
 //!  
 //----------------------------------------------------------------------------
-static bool GetAddRules(Socket & s, const string & secret,
-                        const string & table)
+static bool Ping(Credence::Peer & peer)
+{
+  bool                      rc = false;
+  McBlock::ResponseMessage  resp;
+  nlohmann::json            jv;
+  jv["type"] = "ping";
+  McBlock::RequestMessage   req(jv);
+  if (RequestResponse(peer, req, resp)) {
+    if (! resp.Json().empty()) {
+      if (resp.Json().find("alive") != resp.Json().end()) {
+        if (resp.Json()["alive"].is_boolean()) {
+          if (resp.Json()["alive"].get<bool>()) {
+            cout << "alive\n";
+            rc = true;
+          }
+        }
+      }
+    }
+  }
+  return rc;
+}
+  
+//----------------------------------------------------------------------------
+//!  
+//----------------------------------------------------------------------------
+static bool GetAddRules(Credence::Peer & peer, const string & table)
 {
   bool  rc = false;
   McBlock::ResponseMessage  resp;
   McBlock::RequestMessage   req((McBlock::GetAddRulesReq(table)));
-  if (RequestResponse(s, secret, req, resp)) {
+  if (RequestResponse(peer, req, resp)) {
     if (! resp.Json().empty()) {
-      cout << resp.Json().toStyledString() << '\n';
+      cout << resp.Json().dump(2) << '\n';
       rc = true;
     }
   }
@@ -344,17 +331,17 @@ static bool GetAddRules(Socket & s, const string & secret,
 //----------------------------------------------------------------------------
 //!  
 //----------------------------------------------------------------------------
-static bool LogHit(Socket & s, const string & secret,
-                   const string & table, const string & addrstr)
+static bool LogHit(Credence::Peer & peer, const string & table,
+                   const string & addrstr)
 {
   Dwm::TimeValue64  now(true);
   bool  rc = false;
   Ipv4Address  addr(addrstr);
   McBlock::ResponseMessage  resp;
   McBlock::RequestMessage   req((McBlock::LogHitReq(table, addr, now)));
-  if (RequestResponse(s, secret, req, resp)) {
+  if (RequestResponse(peer, req, resp)) {
     if (! resp.Json().empty()) {
-      cout << resp.Json().toStyledString() << '\n';
+      cout << resp.Json().dump(2) << '\n';
       rc = true;
     }
   }
@@ -364,15 +351,14 @@ static bool LogHit(Socket & s, const string & secret,
 //----------------------------------------------------------------------------
 //!  
 //----------------------------------------------------------------------------
-static bool Get(Socket & s, const string & secret,
-                const string & table, const string & addrstr,
-                Json::Value & jv)
+static bool Get(Credence::Peer & peer, const string & table,
+                const string & addrstr, nlohmann::json & jv)
 {
   bool  rc = false;
   Ipv4Address  addr(addrstr);
   McBlock::ResponseMessage  resp;
   McBlock::RequestMessage   req((McBlock::GetReq(table, addr)));
-  if (RequestResponse(s, secret, req, resp)) {
+  if (RequestResponse(peer, req, resp)) {
     if (! resp.Json().empty()) {
       jv = resp.Json();
       rc = true;
@@ -384,18 +370,18 @@ static bool Get(Socket & s, const string & secret,
 //----------------------------------------------------------------------------
 //!  
 //----------------------------------------------------------------------------
-static bool Edit(Socket & s, const string & secret,
-                const string & table, const string & addrstr)
+static bool Edit(Credence::Peer & peer, const string & table,
+                 const string & addrstr)
 {
-  bool  rc = false;
-  Json::Value  jv;
-  if (Get(s, secret, table, addrstr, jv)) {
-    cout << "Editing prefix " << jv["prefix"].asString() << " in table "
-         << jv["table"].asString() << endl;
+  bool            rc = false;
+  nlohmann::json  jv;
+  if (Get(peer, table, addrstr, jv)) {
+    cout << "Editing prefix " << jv["prefix"].get<string>() << " in table "
+         << jv["table"].get<string>() << endl;
     string  st;
     string  countryCode;
     while (countryCode.empty()) {
-      cout << "countryCode [" << jv["countryCode"].asString() << "]: "
+      cout << "countryCode [" << jv["countryCode"].get<string>() << "]: "
            << std::flush;
       std::getline(cin, st);
       if (! st.empty()) {
@@ -404,17 +390,17 @@ static bool Edit(Socket & s, const string & secret,
         }
       }
       else {
-        countryCode = jv["countryCode"].asString();
+        countryCode = jv["countryCode"].get<string>();
       }
     }
     jv["countryCode"] = countryCode;
   readDaysRemaining:
-    cout << "daysRemaining [" << jv["daysRemaining"].asUInt() << "]: "
+    cout << "daysRemaining [" << jv["daysRemaining"].get<uint32_t>() << "]: "
          << std::flush;
     std::getline(cin, st);
     if (! st.empty()) {
       try {
-        jv["daysRemaining"] = static_cast<Json::UInt>(stoul(st, nullptr, 10));
+        jv["daysRemaining"] = static_cast<uint32_t>(stoul(st, nullptr, 10));
       }
       catch (...) {
         goto readDaysRemaining;
@@ -424,9 +410,9 @@ static bool Edit(Socket & s, const string & secret,
     jv["type"] = "edit";
     McBlock::ResponseMessage  resp;
     McBlock::RequestMessage   req(jv);
-    if (RequestResponse(s, secret, req, resp)) {
+    if (RequestResponse(peer, req, resp)) {
       if (! resp.Json().empty()) {
-        cout << resp.Json().toStyledString() << '\n';
+        cout << resp.Json().dump(2) << '\n';
         rc = true;
       }
     }
@@ -438,16 +424,16 @@ static bool Edit(Socket & s, const string & secret,
 //----------------------------------------------------------------------------
 //!  
 //----------------------------------------------------------------------------
-static bool Get(Socket & s, const string & secret,
-                const string & table, const string & addrstr)
+static bool Get(Credence::Peer & peer, const string & table,
+                const string & addrstr)
 {
   bool  rc = false;
   Ipv4Address  addr(addrstr);
   McBlock::ResponseMessage  resp;
   McBlock::RequestMessage   req((McBlock::GetReq(table, addr)));
-  if (RequestResponse(s, secret, req, resp)) {
+  if (RequestResponse(peer, req, resp)) {
     if (! resp.Json().empty()) {
-      cout << resp.Json().toStyledString() << '\n';
+      cout << resp.Json().dump(2) << '\n';
       rc = true;
     }
   }
@@ -466,7 +452,7 @@ int main(int argc, char *argv[])
   
   int  nextArg = optargs.Parse(argc, argv);
 
-  if (argc < 2) {
+  if (nextArg > (argc - 1)) {
     Usage(argv[0]);
     exit(1);
   }
@@ -488,11 +474,12 @@ int main(int argc, char *argv[])
 
   Signal  sigPipe(SIGPIPE);
   sigPipe.Block();
-  
-  Socket  s;
-  string  theirId, secret;
-  if (Connect(mcblockdHost, s)) {
-    if (! Authenticate(s, theirId, secret)) {
+
+  Credence::Peer       peer;
+  Credence::KeyStash   keyStash;
+  Credence::KnownKeys  knownKeys;
+  if (Connect(mcblockdHost, peer)) {
+    if (! peer.Authenticate(keyStash, knownKeys)) {
       cerr << "Failed to authenticate!\n";
       return 1;
     }
@@ -501,8 +488,13 @@ int main(int argc, char *argv[])
     cerr << "Failed to connect to " << mcblockdHost << '\n';
     return 1;
   }
-  
-  if (string(argv[nextArg]) == "activate") {
+
+  if (string(argv[nextArg]) == "ping") {
+    if (Ping(peer)) {
+      rc = 0;
+    }
+  }
+  else if (string(argv[nextArg]) == "activate") {
     if (argc > ++nextArg) {
       string  table = argv[nextArg];
       if (argc > ++nextArg) {
@@ -510,7 +502,7 @@ int main(int argc, char *argv[])
         for ( ; nextArg < argc; ++nextArg) {
           prefixes.push_back(Ipv4Prefix(argv[nextArg]));
         }
-        if (ActivatePrefixes(s, secret, table, prefixes)) {
+        if (ActivatePrefixes(peer, table, prefixes)) {
           rc = 0;
         }
       }
@@ -532,7 +524,7 @@ int main(int argc, char *argv[])
         for ( ; nextArg < argc; ++nextArg) {
           prefixes.push_back(Ipv4Prefix(argv[nextArg]));
         }
-        if (DeactivatePrefixes(s, secret, table, prefixes)) {
+        if (DeactivatePrefixes(peer, table, prefixes)) {
           rc = 0;
         }
       }
@@ -548,7 +540,7 @@ int main(int argc, char *argv[])
   }
   else if (string(argv[nextArg]) == "search") {
     if (argc > ++nextArg) {
-      if (Search(s, secret, Ipv4Address(argv[nextArg]))) {
+      if (Search(peer, Ipv4Address(argv[nextArg]))) {
         rc = 0;
       }
     }
@@ -562,7 +554,7 @@ int main(int argc, char *argv[])
     if (argc > ++nextArg) {
       table = argv[nextArg];
     }
-    if (GetActivePrefixes(s, secret, table)) {
+    if (GetActivePrefixes(peer, table)) {
       rc = 0;
     }
     else {
@@ -573,7 +565,7 @@ int main(int argc, char *argv[])
   else if (string(argv[nextArg]) == "getaddrules") {
     if (argc > ++nextArg) {
       string  table = argv[nextArg];
-      if (GetAddRules(s, secret, table)) {
+      if (GetAddRules(peer, table)) {
         rc = 0;
       }
     }
@@ -587,7 +579,7 @@ int main(int argc, char *argv[])
       string  table = argv[nextArg];
       if (argc > ++nextArg) {
         string  hostaddr = argv[nextArg];
-        if (LogHit(s, secret, table, hostaddr)) {
+        if (LogHit(peer, table, hostaddr)) {
           rc = 0;
         }
       }
@@ -606,7 +598,7 @@ int main(int argc, char *argv[])
       string  table = argv[nextArg];
       if (argc > ++nextArg) {
         string  hostaddr = argv[nextArg];
-        if (Get(s, secret, table, hostaddr)) {
+        if (Get(peer, table, hostaddr)) {
           rc = 0;
         }
       }
@@ -625,7 +617,7 @@ int main(int argc, char *argv[])
       string  table = argv[nextArg];
       if (argc > ++nextArg) {
         string  hostaddr = argv[nextArg];
-        if (Edit(s, secret, table, hostaddr)) {
+        if (Edit(peer, table, hostaddr)) {
           rc = 0;
         }
       }
