@@ -1,6 +1,6 @@
 //===========================================================================
 // @(#) $Name:$
-// @(#) $Id: DwmMcBlockAddRules.cc 9146 2017-04-18 20:57:42Z dwm $
+// @(#) $Id: DwmMcBlockAddRules.cc 12091 2022-11-29 05:05:27Z dwm $
 //===========================================================================
 //  Copyright (c) Daniel W. McRobb 2017
 //  All rights reserved.
@@ -43,7 +43,7 @@
 #include "DwmSysLogger.hh"
 #include "DwmMcBlockAddRules.hh"
 
-static const Dwm::SvnTag svntag("@(#) $DwmPath: dwm/mcplex/mcblock/tags/mcblock-0.1.1/apps/mcblockd/DwmMcBlockAddRules.cc 9146 $");
+static const Dwm::SvnTag svntag("@(#) $DwmPath: dwm/mcplex/mcblock/tags/mcblock-0.3.3/apps/mcblockd/DwmMcBlockAddRules.cc 12091 $");
 
 using namespace std;
 
@@ -60,32 +60,32 @@ namespace Dwm {
           _rgx(_countries, regex::ECMAScript|regex::optimize),
           _widestMask(24), _days(30), _logThresh(1), _logDays(1)
     {
-      const Json::Value  *jvp =
+      const nlohmann::json  *jvp =
         config.Find("AddRules/" + tableName + "/" + _name);
       if (jvp) {
-        const Json::Value  *p = config.Find(jvp, "countries");
-        if (p && p->isString()) {
-          _countries = p->asString();
+        const nlohmann::json  *p = config.Find(jvp, "countries");
+        if (p && p->is_string()) {
+          _countries = p->get<string>();
         }
         _rgx = regex(_countries, regex::ECMAScript|regex::optimize);
         p = config.Find(jvp, "widestMask");
-        if (p && p->isString()) {
-          _widestMask = stol(p->asString());
+        if (p && p->is_string()) {
+          _widestMask = stol(p->get<string>());
           if (_widestMask > 32) {
             _widestMask = 24;
           }
         }
         p = config.Find(jvp, "days");
-        if (p && p->isString()) {
-          _days = stol(p->asString());
+        if (p && p->is_string()) {
+          _days = stol(p->get<string>());
         }
         p = config.Find(jvp, "logThresh");
-        if (p && p->isString()) {
-          _logThresh = stoul(p->asString());
+        if (p && p->is_string()) {
+          _logThresh = stoul(p->get<string>());
         }
         p = config.Find(jvp, "logDays");
-        if (p && p->isString()) {
-          _logDays = stoul(p->asString());
+        if (p && p->is_string()) {
+          _logDays = stoul(p->get<string>());
         }
       }
     }
@@ -164,9 +164,9 @@ namespace Dwm {
     //------------------------------------------------------------------------
     //!  
     //------------------------------------------------------------------------
-    Json::Value AddRule::Json() const
+    nlohmann::json AddRule::Json() const
     {
-      Json::Value  rc;
+      nlohmann::json  rc;
       rc["countries"] = _countries;
       rc["widestMask"] = to_string((unsigned)_widestMask);
       rc["days"] = to_string(_days);
@@ -199,14 +199,11 @@ namespace Dwm {
                                        const string & tableName)
         : _tableName(tableName)
     {
-      const Json::Value  *jvp = config.Find("AddRules/" + tableName);
-      if (jvp) {
-        if (jvp->size() > 0) {
-          vector<string>  &&ruleNames = jvp->getMemberNames();
-          for (auto name : ruleNames) {
-            AddRule  rule(config, tableName, name);
-            _addRules.push_back(rule);
-          }
+      const nlohmann::json  *jvp = config.Find("AddRules/" + tableName);
+      if (jvp && (jvp->is_object())) {
+        for (auto & [key, value] : jvp->items()) {
+          AddRule  rule(config, tableName, key);
+          _addRules.push_back(rule);
         }
       }
     }
@@ -257,9 +254,9 @@ namespace Dwm {
     //------------------------------------------------------------------------
     //!  
     //------------------------------------------------------------------------
-    Json::Value AddRulesForTable::Json() const
+    nlohmann::json AddRulesForTable::Json() const
     {
-      Json::Value  rc;
+      nlohmann::json  rc;
       for (auto & ar : _addRules) {
         rc[ar.Name()] = ar.Json();
       }
@@ -287,13 +284,12 @@ namespace Dwm {
     //------------------------------------------------------------------------
     AddRules::AddRules(const Cfg2Json & config)
     {
-      const Json::Value  *jvp = config.Find("AddRules");
+      const nlohmann::json  *jvp = config.Find("AddRules");
       if (jvp) {
-        if (jvp->size() > 0) {
-          vector<string>  &&tableNames = jvp->getMemberNames();
-          for (auto tableName : tableNames) {
-            AddRulesForTable  rulesForTable(config, tableName);
-            _addRules[tableName] = rulesForTable;
+        if (jvp && jvp->is_object()) {
+          for (auto & [key, value] : jvp->items()) {
+            AddRulesForTable  rulesForTable(config, key);
+            _addRules[key] = rulesForTable;
           }
         }
       }
@@ -319,9 +315,9 @@ namespace Dwm {
     //------------------------------------------------------------------------
     //!  
     //------------------------------------------------------------------------
-    Json::Value AddRules::Json() const
+    nlohmann::json AddRules::Json() const
     {
-      Json::Value  rc;
+      nlohmann::json  rc;
       for (auto & art : _addRules) {
         rc["AddRules"][art.second.Name()] = art.second.Json();
       }
