@@ -1,8 +1,8 @@
 //===========================================================================
 // @(#) $Name:$
-// @(#) $Id: DwmMcBlockResponder.cc 9688 2017-06-30 09:25:29Z dwm $
+// @(#) $Id: DwmMcBlockResponder.cc 12115 2022-12-06 03:41:24Z dwm $
 //===========================================================================
-//  Copyright (c) Daniel W. McRobb 2017
+//  Copyright (c) Daniel W. McRobb 2017, 2022
 //  All rights reserved.
 //
 //  Redistribution and use in source and binary forms, with or without
@@ -54,15 +54,13 @@ extern "C" {
 
 #include "DwmSysLogger.hh"
 #include "DwmSvnTag.hh"
-#include "DwmAuth.hh"
-#include "DwmAuthPeerAuthenticator.hh"
 #include "DwmMcBlockPortability.hh"
 #include "DwmMcBlockRequestMessage.hh"
 #include "DwmMcBlockResponseMessage.hh"
 #include "DwmMcBlockServer.hh"
 #include "DwmMcBlockTcpDrop.hh"
 
-static const Dwm::SvnTag  svntag("$DwmPath: dwm/mcplex/mcblock/tags/mcblock-0.1.1/apps/mcblockd/DwmMcBlockResponder.cc 9688 $");
+static const Dwm::SvnTag  svntag("$DwmPath: dwm/mcplex/mcblock/tags/mcblock-0.3.3/apps/mcblockd/DwmMcBlockResponder.cc 12115 $");
 
 using namespace std;
 
@@ -73,15 +71,21 @@ namespace Dwm {
     //------------------------------------------------------------------------
     //!  
     //------------------------------------------------------------------------
-    Responder::Responder(Server & server)
-        : _socket(), _theirId(), _server(server)
+    Responder::Responder(boost::asio::ip::tcp::socket && s, Server & server)
+        : _peer(), _server(server), _running(false)
     {
-      if (_server.GetSocket().Accept(_socket, _clientAddr)) {
-        Syslog(LOG_INFO, "Accepted connection from %s:%hu",
-               inet_ntoa(_clientAddr.sin_addr), ntohs(_clientAddr.sin_port));
+      boost::system::error_code  ec;
+      s.native_non_blocking(false, ec);
+      _peer.SetKeyExchangeTimeout(chrono::milliseconds(2000));
+      _peer.SetIdExchangeTimeout(chrono::milliseconds(2000));
+      if (_peer.Accept(std::move(s))) {
         _run = true;
         _thread = std::thread(&Responder::Run, this);
-        _running = true;
+        _running.store(true);
+      }
+      else {
+        _peer.Disconnect();
+        Syslog(LOG_ERR, "Failed to accept peer");
       }
     }
 
@@ -100,9 +104,11 @@ namespace Dwm {
     bool Responder::Join()
     {
       bool  rc = false;
-      if (_thread.joinable() && (! _running)) {
-        _thread.join();
-        rc = true;
+      if (! _running.load()) {
+        if (_thread.joinable()) {
+          _thread.join();
+          rc = true;
+        }
       }
       return rc;
     }
@@ -110,124 +116,135 @@ namespace Dwm {
     //------------------------------------------------------------------------
     //!  
     //------------------------------------------------------------------------
-    bool Responder::IsValidGetRequest(const Json::Value & request)
+    bool Responder::IsValidGetRequest(const nlohmann::json & request)
     {
-      return (request.isMember("type")
-              && (request["type"].isString())
-              && (request["type"] == "get")
-              && (request.isMember("table"))
-              && (request["table"].isString())
-              && (request.isMember("ipv4addr"))
-              && (request["ipv4addr"].isString()));
+      return ((request.find("type") != request.end())
+              && (request["type"].is_string())
+              && (request["type"].get<string>() == "get")
+              && (request.find("table") != request.end())
+              && (request["table"].is_string())
+              && (request.find("ipv4addr") != request.end())
+              && (request["ipv4addr"].is_string()));
     }
 
     //------------------------------------------------------------------------
     //!  
     //------------------------------------------------------------------------
-    bool Responder::IsValidEditRequest(const Json::Value & request)
+    bool Responder::IsValidEditRequest(const nlohmann::json & request)
     {
-      return (request.isMember("type")
-              && request["type"].isString()
-              && (request["type"] == "edit")
-              && request.isMember("table")
-              && request["table"].isString()
-              && request.isMember("prefix")
-              && request["prefix"].isString()
-              && request.isMember("countryCode")
-              && request["countryCode"].isString()
-              && (request["countryCode"].asString().size() == 2)
-              && request.isMember("daysRemaining")
-              && request["daysRemaining"].isUInt());
+      return ((request.find("type") != request.end())
+              && request["type"].is_string()
+              && (request["type"].get<string>() == "edit")
+              && (request.find("table") != request.end())
+              && request["table"].is_string()
+              && (request.find("prefix") != request.end())
+              && request["prefix"].is_string()
+              && (request.find("countryCode") != request.end())
+              && request["countryCode"].is_string()
+              && (request["countryCode"].get<string>().size() == 2)
+              && (request.find("daysRemaining") != request.end())
+              && request["daysRemaining"].is_number_unsigned());
     }
     
     //------------------------------------------------------------------------
     //!  
     //------------------------------------------------------------------------
-    bool Responder::IsValidActivateRequest(const Json::Value & request)
+    bool Responder::IsValidActivateRequest(const nlohmann::json & request)
     {
-      return ((request.isMember("type"))
-              && (request["type"].isString())
-              && (request["type"] == "activate")
-              && (request.isMember("table"))
-              && (request["table"].isString())
-              && (request.isMember("prefixes"))
-              && (request["prefixes"].isArray())
+      return ((request.find("type") != request.end())
+              && (request["type"].is_string())
+              && (request["type"].get<string>() == "activate")
+              && (request.find("table") != request.end())
+              && (request["table"].is_string())
+              && (request.find("prefixes") != request.end())
+              && (request["prefixes"].is_array())
               && (request["prefixes"].size() > 0)
-              && (request["prefixes"][0].isString()));
+              && (request["prefixes"][0].is_string()));
     }
 
     //------------------------------------------------------------------------
     //!  
     //------------------------------------------------------------------------
-    bool Responder::IsValidDeactivateRequest(const Json::Value & request)
+    bool Responder::IsValidDeactivateRequest(const nlohmann::json & request)
     {
-      return ((request.isMember("type"))
-              && (request["type"].isString())
-              && (request["type"].asString() == "deactivate")
-              && (request.isMember("table"))
-              && (request["table"].isString())
-              && (request.isMember("prefixes"))
-              && (request["prefixes"].isArray())
+      return ((request.find("type") != request.end())
+              && (request["type"].is_string())
+              && (request["type"].get<string>() == "deactivate")
+              && (request.find("table") != request.end())
+              && (request["table"].is_string())
+              && (request.find("prefixes") != request.end())
+              && (request["prefixes"].is_array())
               && (request["prefixes"].size() > 0)
-              && (request["prefixes"][0].isString()));
+              && (request["prefixes"][0].is_string()));
     }
 
     //------------------------------------------------------------------------
     //!  
     //------------------------------------------------------------------------
-    bool Responder::IsValidGetActiveRequest(const Json::Value & request)
+    bool Responder::IsValidGetActiveRequest(const nlohmann::json & request)
     {
       return ((! request.empty())
-              && (request.isMember("type"))
-              && (request["type"].isString())
-              && (request["type"].asString() == "getActive")
-              && (request.isMember("table"))
-              && (request["table"].isString()));
+              && (request.find("type") != request.end())
+              && (request["type"].is_string())
+              && (request["type"].get<string>() == "getActive")
+              && (request.find("table") != request.end())
+              && (request["table"].is_string()));
     }
 
     //------------------------------------------------------------------------
     //!  
     //------------------------------------------------------------------------
-    bool Responder::IsValidSearchRequest(const Json::Value & request)
+    bool Responder::IsValidSearchRequest(const nlohmann::json & request)
     {
       return ((! request.empty())
-              && (request.isMember("type"))
-              && (request["type"].isString())
-              && (request["type"].asString() == "search")
-              && (request.isMember("ipv4addr"))
-              && (request["ipv4addr"].isString()));
+              && (request.find("type") != request.end())
+              && (request["type"].is_string())
+              && (request["type"].get<string>() == "search")
+              && (request.find("ipv4addr") != request.end())
+              && (request["ipv4addr"].is_string()));
     }
     
     //------------------------------------------------------------------------
     //!  
     //------------------------------------------------------------------------
-    bool Responder::IsValidGetAddRulesRequest(const Json::Value & request)
+    bool Responder::IsValidGetAddRulesRequest(const nlohmann::json & request)
     {
       return ((! request.empty())
-              && (request.isMember("type"))
-              && (request["type"].isString())
-              && (request["type"].asString() == "getAddRules")
-              && (request.isMember("table"))
-              && (request["table"].isString()));
+              && (request.find("type") != request.end())
+              && (request["type"].is_string())
+              && (request["type"].get<string>() == "getAddRules")
+              && (request.find("table") != request.end())
+              && (request["table"].is_string()));
     }
 
     //------------------------------------------------------------------------
     //!  
     //------------------------------------------------------------------------
-    bool Responder::IsValidLogHitRequest(const Json::Value & request)
+    bool Responder::IsValidLogHitRequest(const nlohmann::json & request)
     {
       return ((! request.empty())
-              && (request.isMember("type"))
-              && (request["type"].isString())
-              && (request["type"].asString() == "logHit")
-              && (request.isMember("table"))
-              && (request["table"].isString())
-              && (request.isMember("ipv4addr"))
-              && (request["ipv4addr"].isString())
-              && (request.isMember("logTime"))
-              && (request["logTime"].isUInt()));
+              && (request.find("type") != request.end())
+              && (request["type"].is_string())
+              && (request["type"].get<string>() == "logHit")
+              && (request.find("table") != request.end())
+              && (request["table"].is_string())
+              && (request.find("ipv4addr") != request.end())
+              && (request["ipv4addr"].is_string())
+              && (request.find("logTime") != request.end())
+              && (request["logTime"].is_number_unsigned()));
     }
 
+    //------------------------------------------------------------------------
+    //!  
+    //------------------------------------------------------------------------
+    bool Responder::IsValidPingRequest(const nlohmann::json & request)
+    {
+      return ((! request.empty())
+              && (request.find("type") != request.end())
+              && (request["type"].is_string())
+              && (request["type"].get<string>() == "ping"));
+    }
+      
     //------------------------------------------------------------------------
     //!  
     //------------------------------------------------------------------------
@@ -235,10 +252,11 @@ namespace Dwm {
     Responder::GetRDAPEntries(const vector<Ipv4Address> & addrs)
     {
       RDAP::FetchedEntryMap  rc;
+      Credence::KeyStash     keyStash(_server.Config().KeyDirectory());
+      Credence::KnownKeys    knownKeys(_server.Config().KeyDirectory());
       RDAP::Fetcher          fetcher;
       if (fetcher.OpenSession(_server.Config().DwmRDAPServer(),
-                              _server.Config().DwmRDAPPrivateKeyFile(),
-                              _server.Config().KnownServicesFile())) {
+                              keyStash, knownKeys)) {
         rc = fetcher.GetEntries(addrs);
         fetcher.CloseSession();
       }
@@ -248,18 +266,19 @@ namespace Dwm {
     //------------------------------------------------------------------------
     //!  
     //------------------------------------------------------------------------
-    Json::Value Responder::HandleActivateRequest(const Json::Value & request)
+    nlohmann::json
+    Responder::HandleActivateRequest(const nlohmann::json & request)
     {
-      Json::FastWriter  fw;
-      Syslog(LOG_INFO, "request %s", fw.write(request).c_str());
-      Json::Value  rc;
+      Syslog(LOG_DEBUG, "request %s", request.dump().c_str());
+      nlohmann::json       rc;
       vector<Ipv4Prefix>   prefixes;
       vector<Ipv4Address>  addrs;
       
-      Pf::Table  &&table = _server.GetPfDevice().GetTable("", request["table"].asString());
-      if (table.Name() == request["table"].asString()) {
+      Pf::Table  &&table =
+        _server.GetPfDevice().GetTable("", request["table"].get<string>());
+      if (table.Name() == request["table"].get<string>()) {
         for (uint32_t i = 0; i < request["prefixes"].size(); ++i) {
-          Ipv4Prefix  prefix(request["prefixes"][i].asString());
+          Ipv4Prefix  prefix(request["prefixes"][i].get<string>());
           prefixes.push_back(prefix);
           if (prefix.MaskLength() == 32) {
             addrs.push_back(prefix.FirstAddress());
@@ -282,6 +301,10 @@ namespace Dwm {
               rc["prefixes"][ai]["requested"] = request["prefixes"][ai];
               rc["prefixes"][ai]["added"] = match.ToShortString();
             }
+            else {
+              Syslog(LOG_ERR, "Failed to add %s to %s",
+                     prefixes[ai].ToString().c_str(), table.Name().c_str());
+            }
           }
           else {
             rc["table"] = request["table"];
@@ -290,19 +313,26 @@ namespace Dwm {
           }
         }
       }
+      else {
+        Syslog(LOG_ERR, "Table '%s' not found",
+               request["table"].get<string>().c_str());
+      }
+      
       return rc;
     }
 
     //------------------------------------------------------------------------
     //!  
     //------------------------------------------------------------------------
-    Json::Value Responder::HandleDeactivateRequest(const Json::Value & request)
+    nlohmann::json
+    Responder::HandleDeactivateRequest(const nlohmann::json & request)
     {
-      Json::Value  rc;
-      Pf::Table  &&table = _server.GetPfDevice().GetTable("", request["table"].asString());
-      if (table.Name() == request["table"].asString()) {
+      nlohmann::json    rc;
+      Pf::Table       &&table =
+        _server.GetPfDevice().GetTable("", request["table"].get<string>());
+      if (table.Name() == request["table"].get<string>()) {
         for (uint32_t i = 0; i < request["prefixes"].size(); ++i) {
-          Ipv4Prefix  prefix(request["prefixes"][i].asString());
+          Ipv4Prefix  prefix(request["prefixes"][i].get<string>());
           Ipv4Prefix  match(prefix);
           if (table.Contains(prefix, match)) {
             if (match == prefix) {
@@ -325,10 +355,11 @@ namespace Dwm {
     //------------------------------------------------------------------------
     //!  
     //------------------------------------------------------------------------
-    Json::Value Responder::HandleGetActiveRequest(const Json::Value & request)
+    nlohmann::json
+    Responder::HandleGetActiveRequest(const nlohmann::json & request)
     {
-      Json::Value  rc;
-      string       tableName = request["table"].asString();
+      nlohmann::json  rc;
+      string          tableName = request["table"].get<string>();
       auto  dbi = _server.GetDatabases().begin();
       if (! tableName.empty()) {
         dbi = _server.GetDatabases().find(tableName);
@@ -364,11 +395,12 @@ namespace Dwm {
     //------------------------------------------------------------------------
     //!  
     //------------------------------------------------------------------------
-    Json::Value Responder::HandleGetRequest(const Json::Value & request)
+    nlohmann::json
+    Responder::HandleGetRequest(const nlohmann::json & request)
     {
-      Json::Value  rc;
-      Ipv4Address  addr(request["ipv4addr"].asString());
-      auto dbi = _server.GetDatabases().find(request["table"].asString());
+      nlohmann::json  rc;
+      Ipv4Address     addr(request["ipv4addr"].get<string>());
+      auto dbi = _server.GetDatabases().find(request["table"].get<string>());
       if (dbi != _server.GetDatabases().end()) {
         pair<Ipv4Prefix,DbEntry>  match;
         if (dbi->second.FindLongest(addr, match)) {
@@ -388,21 +420,22 @@ namespace Dwm {
     //------------------------------------------------------------------------
     //!  
     //------------------------------------------------------------------------
-    Json::Value Responder::HandleEditRequest(const Json::Value & request)
+    nlohmann::json
+    Responder::HandleEditRequest(const nlohmann::json & request)
     {
-      Json::Value  rc;
-      Ipv4Prefix   pfx(request["prefix"].asString());
-      auto dbi = _server.GetDatabases().find(request["table"].asString());
+      nlohmann::json  rc;
+      Ipv4Prefix      pfx(request["prefix"].get<string>());
+      auto dbi = _server.GetDatabases().find(request["table"].get<string>());
       if (dbi != _server.GetDatabases().end()) {
         DbEntry  match;
         if (dbi->second.Find(pfx, match)) {
           TimeValue  now(true);
           TimeValue  endTime(now);
           endTime.Set(endTime.Secs()
-                      + (request["daysRemaining"].asUInt() * 24 * 60 * 60), 0);
+                      + (request["daysRemaining"].get<uint64_t>() * 24 * 60 * 60), 0);
           TimeInterval  interval(match.Interval().Start(), endTime);
           match.Interval(interval);
-          match.Country(request["countryCode"].asString());
+          match.Country(request["countryCode"].get<string>());
           dbi->second[pfx] = match;
           dbi->second.Save("");
           rc["edited"] = true;
@@ -421,11 +454,12 @@ namespace Dwm {
     //------------------------------------------------------------------------
     //!  
     //------------------------------------------------------------------------
-    Json::Value Responder::HandleSearchRequest(const Json::Value & request)
+    nlohmann::json
+    Responder::HandleSearchRequest(const nlohmann::json & request)
     {
-      Json::Value  rc;
-      Ipv4Address  addr(request["ipv4addr"].asString());
-      TimeValue  now(true);
+      nlohmann::json  rc;
+      Ipv4Address     addr(request["ipv4addr"].get<string>());
+      TimeValue       now(true);
       for (const auto & db : _server.GetDatabases()) {
         pair<Ipv4Prefix,DbEntry>  match;
         if (db.second.FindLongest(addr, match)) {
@@ -447,17 +481,29 @@ namespace Dwm {
     //------------------------------------------------------------------------
     //!  
     //------------------------------------------------------------------------
-    Json::Value
-    Responder::HandleGetAddRulesRequest(const Json::Value & request)
+    nlohmann::json
+    Responder::HandleGetAddRulesRequest(const nlohmann::json & request)
     {
-      Json::Value  rc;
-      auto  ari = _server.GetAddRules().RulesForTable(request["table"].asString());
+      nlohmann::json  rc;
+      auto  ari =
+        _server.GetAddRules().RulesForTable(request["table"].get<string>());
       if (ari != _server.GetAddRules().RulesForTables().end()) {
         rc = ari->second.Json();
       }
       return rc;
     }
 
+    //------------------------------------------------------------------------
+    //!  
+    //------------------------------------------------------------------------
+    nlohmann::json
+    Responder::HandlePingRequest(const nlohmann::json & request)
+    {
+      nlohmann::json  rc;
+      rc["alive"] = true;
+      return rc;
+    }
+    
     //------------------------------------------------------------------------
     //!  
     //------------------------------------------------------------------------
@@ -466,9 +512,11 @@ namespace Dwm {
     {
       RDAP::Fetcher                  fetcher;
       pair<bool,RDAP::FetchedEntry>  rdapResp;  rdapResp.first = false;
+      Credence::KeyStash             keyStash(_server.Config().KeyDirectory());
+      Credence::KnownKeys            knownKeys(_server.Config().KeyDirectory());
+      
       if (fetcher.OpenSession(_server.Config().DwmRDAPServer(),
-                              _server.Config().DwmRDAPPrivateKeyFile(),
-                              _server.Config().KnownServicesFile())) {
+                              keyStash, knownKeys)) {
         rdapResp = fetcher.GetEntry(addr);
         fetcher.CloseSession();
       }
@@ -554,12 +602,14 @@ namespace Dwm {
     //------------------------------------------------------------------------
     //!  This is gnarly.  How can I refactor it to make it easier to maintain?
     //------------------------------------------------------------------------
-    Json::Value Responder::HandleLogHitRequest(const Json::Value & request)
+    nlohmann::json
+    Responder::HandleLogHitRequest(const nlohmann::json & request)
     {
-      Json::Value  rc;
-      Ipv4Address  addr(request["ipv4addr"].asString());
-      Pf::Table  &&pftable = _server.GetPfDevice().GetTable("", request["table"].asString());
-      if (pftable.Name() == request["table"].asString()) {
+      nlohmann::json  rc;
+      Ipv4Address     addr(request["ipv4addr"].get<string>());
+      Pf::Table  &&pftable =
+        _server.GetPfDevice().GetTable("", request["table"].get<string>());
+      if (pftable.Name() == request["table"].get<string>()) {
         Ipv4Prefix  match;
         if (pftable.Contains(addr, match)) {
           // already blocked
@@ -576,7 +626,8 @@ namespace Dwm {
 
           AddRule  addRule;
           //  Get the rules for the table
-          auto  ari = _server.GetAddRules().RulesForTable(request["table"].asString());
+          auto  ari =
+            _server.GetAddRules().RulesForTable(request["table"].get<string>());
           if (ari != _server.GetAddRules().RulesForTables().end()) {
             //  And the rule for the country
             if (ari->second.FindRuleForCountry(country, addRule)) {
@@ -603,8 +654,8 @@ namespace Dwm {
               }
             }
             else {
-              TimeValue64  logtime(request["logTime"].asUInt64(), 0);
-              auto  leti = _server.GetLogEntryTrackers().find(request["table"].asString());
+              TimeValue64  logtime(request["logTime"].get<uint64_t>(), 0);
+              auto  leti = _server.GetLogEntryTrackers().find(request["table"].get<string>());
               if (leti != _server.GetLogEntryTrackers().end()) {
                 leti->second.Add(prefix, country, logtime);
                 if (leti->second.HitThreshold(prefix, ari->second)) {
@@ -636,7 +687,7 @@ namespace Dwm {
               else {
                 //  No log tracker found, add it
                 TableLogEntryTracker  & letRef =
-                  _server.GetLogEntryTrackers()[request["table"].asString()];
+                  _server.GetLogEntryTrackers()[request["table"].get<string>()];
                 letRef.Add(prefix, country, logtime);
                 Syslog(LOG_INFO, "Pending %s (%s) for %s, %u/%u",
                        prefix.ToShortString().c_str(), country.c_str(),
@@ -655,12 +706,14 @@ namespace Dwm {
       }
       return rc;
     }
-      
+
     //------------------------------------------------------------------------
     //!  
     //------------------------------------------------------------------------
-    Json::Value Responder::HandleRequest(const Json::Value & request)
+    nlohmann::json Responder::HandleRequest(const nlohmann::json & request)
     {
+      Syslog(LOG_DEBUG, "Handling request '%s'", request.dump().c_str());
+      
       if (IsValidActivateRequest(request)) {
         return HandleActivateRequest(request);
       }
@@ -685,9 +738,12 @@ namespace Dwm {
       else if (IsValidEditRequest(request)) {
         return HandleEditRequest(request);
       }
+      else if (IsValidPingRequest(request)) {
+        return HandlePingRequest(request);
+      }
       else {
         Syslog(LOG_ERR, "Invalid request");
-        return Json::Value();
+        return nlohmann::json();
       }
     }
 
@@ -696,47 +752,38 @@ namespace Dwm {
     //------------------------------------------------------------------------
     void Responder::Run()
     {
+#if 0
       set_pthread_name(_thread.native_handle(), "responder");
-      Syslog(LOG_DEBUG, "Responder started on fd %d", (int)_socket);
-
-      int  noDelay = 1;
-      _socket.Setsockopt(IPPROTO_TCP, TCP_NODELAY, &noDelay,
-                         sizeof(noDelay));
-      RequestMessage  reqmsg;
-      Auth::PeerAuthenticator  peerAuth(_server.Config().PrivateKeyFile(),
-                                        _server.Config().AuthorizedKeysFile());
-      if (peerAuth.Authenticate(_socket, _theirId, _agreedKey)) {
-        Syslog(LOG_INFO, "Authenticated client %s from %s:%hu",
-               _theirId.c_str(), inet_ntoa(_clientAddr.sin_addr),
-               ntohs(_clientAddr.sin_port));
-        while (_run && reqmsg.Read(_socket, _agreedKey)) {
-          Json::Value  reqJson = reqmsg.Json();
-          ResponseMessage  respmsg(HandleRequest(reqJson));
-          if (! respmsg.Write(_socket, _agreedKey)) {
-            Syslog(LOG_ERR, "Failed to send response message to %s:%hu",
-                   inet_ntoa(_clientAddr.sin_addr),
-                   ntohs(_clientAddr.sin_port));
+#endif
+      Syslog(LOG_DEBUG, "Responder started for %s",
+             _peer.EndPointString().c_str());
+      if (_peer.Authenticate(_server.GetKeyStash(), _server.GetKnownKeys())) {
+        RequestMessage  reqmsg;
+        while (_run) {
+          if (reqmsg.Read(_peer)) {
+            ResponseMessage  respmsg(HandleRequest(reqmsg.Json()));
+            if (! respmsg.Write(_peer)) {
+              Syslog(LOG_ERR, "Failed to send response message to %s at %s",
+                     _peer.Id().c_str(), _peer.EndPointString().c_str());
+              break;
+            }
+          }
+          else {
+            Syslog(LOG_ERR, "Failed to receive message from %s at %s",
+                   _peer.Id().c_str(), _peer.EndPointString().c_str());
             break;
           }
         }
-        Syslog(LOG_INFO, "Done with client %s from %s:%hu",
-               _theirId.c_str(), inet_ntoa(_clientAddr.sin_addr),
-               ntohs(_clientAddr.sin_port));
+        Syslog(LOG_INFO, "Done with client %s at %s",
+               _peer.Id().c_str(), _peer.EndPointString().c_str());
       }
-      else {
-        Syslog(LOG_INFO, "Authentication failed for client from %s:%hu",
-               inet_ntoa(_clientAddr.sin_addr), ntohs(_clientAddr.sin_port));
-      }
-
-      Syslog(LOG_DEBUG, "Responder done for client %s:%hu",
-             inet_ntoa(_clientAddr.sin_addr), ntohs(_clientAddr.sin_port));
-
-      _agreedKey.assign(_agreedKey.size(), '\0');
+      _peer.Disconnect();
+      Syslog(LOG_DEBUG, "Responder done for %s",
+             _peer.EndPointString().c_str());
       _running = false;
-      
       return;
     }
-
+    
   }  // namespace McBlock
 
 }  // namespace Dwm
