@@ -1,6 +1,6 @@
 //===========================================================================
 // @(#) $Name:$
-// @(#) $Id: DwmMcBlockServer.cc 9443 2017-06-06 00:26:17Z dwm $
+// @(#) $Id: DwmMcBlockServer.cc 12269 2023-10-27 02:50:30Z dwm $
 //===========================================================================
 //  Copyright (c) Daniel W. McRobb 2017
 //  All rights reserved.
@@ -52,16 +52,18 @@ extern "C" {
 #include <fstream>
 #include <sstream>
 
-#include "DwmSocket.hh"
+#include <boost/asio.hpp>
+
 #include "DwmSvnTag.hh"
 #include "DwmSysLogger.hh"
 #include "DwmTimeValue64.hh"
 #include "DwmMcBlockPortability.hh"
 #include "DwmMcBlockServer.hh"
-#include "DwmAuthSymCryptoMessage.hh"
+#undef v4
+
 #include "DwmMcBlockResponseMessage.hh"
 
-static const Dwm::SvnTag  svntag("@(#) $DwmPath: dwm/mcplex/mcblock/tags/mcblock-0.1.1/apps/mcblockd/DwmMcBlockServer.cc 9443 $");
+static const Dwm::SvnTag  svntag("@(#) $DwmPath: dwm/mcplex/mcblock/tags/mcblock-0.3.3/apps/mcblockd/DwmMcBlockServer.cc 12269 $");
 
 using namespace std;
 
@@ -73,8 +75,10 @@ namespace Dwm {
     //!  
     //------------------------------------------------------------------------
     Server::Server(const Cfg2Json & config)
-        :  _config(config), _addRules(config), _pfdev(_config.Device()),
-           _socket(), _mcastSocket(), _run(false), _thread(), _responders(),
+        : _config(config), _addRules(config), _pfdev(_config.Device()),
+          _keyStash(_config.KeyDirectory()),
+          _knownKeys(_config.KeyDirectory()),
+          _run(false), _acceptors(), _thread(), _responders(),
           _logEntryTrackers(), _databases()
     {
       for (auto & rft : _addRules.RulesForTables()) {
@@ -83,8 +87,32 @@ namespace Dwm {
         _databases[rft.first].Load(dbFileName);
       }
       _nextListSaveTime = chrono::system_clock::now() + chrono::hours(24);
-    }
 
+      namespace ip = boost::asio::ip;
+      using boost::asio::ip::tcp;
+      boost::system::error_code  ec;
+      ip::address  addr =
+        ip::address::from_string((std::string)_config.TCPAddr(), ec);
+      Syslog(LOG_INFO, "Server address %s",
+             ((std::string)(_config.TCPAddr())).c_str());
+      if (! ec) {
+        tcp::endpoint  ep(addr, _config.TCPPort());
+        ip::tcp::acceptor  acc(_ioContext.get_executor());
+        acc.open(tcp::v4());
+        boost::asio::socket_base::reuse_address option(true);
+        acc.set_option(option);
+        acc.bind(ep);
+        auto & a = _acceptors.emplace_back(std::move(acc));
+        a.listen(20);
+        AcceptLoop(a);
+      }
+      else {
+        Syslog(LOG_ERR, "Bad TCP Address %s",
+               ((std::string)(_config.TCPAddr())).c_str());
+        exit(1);
+      }
+    }
+    
     //------------------------------------------------------------------------
     //!  
     //------------------------------------------------------------------------
@@ -109,14 +137,6 @@ namespace Dwm {
       return _pfdev;
     }
 
-    //------------------------------------------------------------------------
-    //!  
-    //------------------------------------------------------------------------
-    Socket & Server::GetSocket()
-    {
-      return _socket;
-    }
-    
     //------------------------------------------------------------------------
     //!  
     //------------------------------------------------------------------------
@@ -132,43 +152,15 @@ namespace Dwm {
     {
       return _databases;
     }
-    
+
     //------------------------------------------------------------------------
     //!  
     //------------------------------------------------------------------------
     bool Server::Start()
     {
-      bool  rc = false;
-      if (_socket.Open(PF_INET, SOCK_STREAM, 0)) {
-        int  on = 1;
-        _socket.Setsockopt(SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on));
-        _socket.Setsockopt(SOL_SOCKET, SO_REUSEPORT, &on, sizeof(on));
-
-        if (_socket.Bind(_config.TCPAddr(), _config.TCPPort())) {
-          if (_mcastSocket.Open(PF_INET, SOCK_DGRAM, 0)) {
-            if (_socket.Listen(10)) {
-              rc = true;
-              _run = true;
-              _thread = std::thread(&Server::Run, this);
-            }
-            else {
-              Syslog(LOG_ERR, "Listen(10) failed! %m");
-            }
-          }
-          else {
-            Syslog(LOG_ERR, "Failed to open UDP socket for multicast send");
-            _socket.Close();
-          }
-        }
-        else {
-          Syslog(LOG_ERR, "socket.Bind(%s,%hu) failed! %m",
-                 ((string)_config.TCPAddr()).c_str(), _config.TCPPort());
-        }
-      }
-      else {
-        Syslog(LOG_ERR, "socket.Open() failed! %m");
-      }
-      return rc;
+      _run = true;
+      _thread = std::thread(&Server::Run, this);
+      return true;
     }
 
     //------------------------------------------------------------------------
@@ -176,12 +168,13 @@ namespace Dwm {
     //------------------------------------------------------------------------
     void Server::StopResponders()
     {
-      auto  ri = _responders.begin();
-      for ( ; ri != _responders.end(); ++ri) {
-        (*ri)->Stop();
-        Responder  *respToDel = *ri;
-        _responders.erase(ri);
-        delete respToDel;
+      for (auto ri = _responders.begin(); ri != _responders.end(); ) {
+        if ((*ri)->Stop()) {
+          ri = _responders.erase(ri++);
+        }
+        else {
+          ++ri;
+        }
       }
       return;
     }
@@ -191,11 +184,15 @@ namespace Dwm {
     //------------------------------------------------------------------------
     void Server::Stop()
     {
+      Syslog(LOG_INFO, "Stopping server");
       _run = false;
-      StopResponders();
+      // StopResponders();
+      for (auto & a : _acceptors) {
+        a.cancel();
+      }
+      Syslog(LOG_INFO, "Server stopped");
       if (_thread.joinable()) {
         _thread.join();
-        Syslog(LOG_INFO, "Server stopped");
       }
       SavePfLists();
       return;
@@ -206,13 +203,12 @@ namespace Dwm {
     //------------------------------------------------------------------------
     void Server::CleanupResponders()
     {
-      auto  ri = _responders.begin();
-      for ( ; ri != _responders.end(); ++ri) {
+      for (auto ri = _responders.begin(); ri != _responders.end(); ) {
         if ((*ri)->Join()) {
-          Responder  *respToDel = *ri;
-          _responders.erase(ri);
-          delete respToDel;
-          break;
+          ri = _responders.erase(ri);
+        }
+        else {
+          ++ri;
         }
       }
       return;
@@ -231,50 +227,26 @@ namespace Dwm {
       }
       return;
     }
-    
+
     //------------------------------------------------------------------------
     //!  
     //------------------------------------------------------------------------
     void Server::Run()
     {
+      Syslog(LOG_INFO, "Server thread started");
       set_pthread_name(_thread.native_handle(), "server");
-      
-      Syslog(LOG_INFO, "Server started");
       _pfdev.Open();
       CleanupInactiveDbEntries();
       SavePfLists();
 
       while (_run) {
-        struct timeval  timeout;
-        timeout.tv_sec = 0;
-        timeout.tv_usec = 200000;
-        fd_set  readFdSet, errFdSet;
-        FD_ZERO(&readFdSet);
-        FD_ZERO(&errFdSet);
-        _socket.FdSet(readFdSet);
-        _socket.FdSet(errFdSet);
-        if (select(FD_SETSIZE, &readFdSet, 0, &errFdSet, &timeout) > 0) {
-          if (_socket.FdIsSet(readFdSet)) {
-            _responders.push_back(new Responder(*this));
-          }
-        }
-#if 0
-        SendMcastStatus();
-#endif
-        CleanupResponders();
-        CleanupLogTrackers();
-        
-        chrono::system_clock::time_point  now = chrono::system_clock::now();
-        if (now > _nextListSaveTime) {
-          CleanupInactiveDbEntries();
-          SavePfLists();
-          _nextListSaveTime = now + chrono::hours(24);
-        }
+        _ioContext.run();
+        this_thread::sleep_for(chrono::milliseconds(100));
       }
-      _pfdev.Close();
+      Syslog(LOG_INFO, "Server thread done");
       return;
     }
-
+    
     //------------------------------------------------------------------------
     //!  
     //------------------------------------------------------------------------
@@ -316,7 +288,29 @@ namespace Dwm {
       }
       return;
     }
-    
+
+    //------------------------------------------------------------------------
+    //!  
+    //------------------------------------------------------------------------
+    void Server::AcceptLoop(boost::asio::ip::tcp::acceptor & a)
+    {
+      using boost::asio::ip::tcp;
+      
+      a.async_accept(a.get_executor(),
+                     [this,&a](error_code ec, tcp::socket && s)
+                     {
+                       CleanupResponders();
+                       if (! ec) {
+                         _responders.push_back(std::make_shared<Responder>(std::move(s), *this));
+                         AcceptLoop(a);
+                       }
+                       else {
+                         Syslog(LOG_ERR, "Exiting AcceptLoop(): %s",
+                                ec.message().c_str());
+                       }
+                     });
+    }
+
 #if 0
     //------------------------------------------------------------------------
     //!  
