@@ -1,6 +1,6 @@
 //===========================================================================
 // @(#) $Name:$
-// @(#) $Id: DwmMcBlockTcpDrop.cc 9420 2017-06-04 05:28:54Z dwm $
+// @(#) $Id: DwmMcBlockTcpDrop.cc 12157 2023-05-21 17:47:33Z dwm $
 //===========================================================================
 //  Copyright (c) Daniel W. McRobb 2017
 //  All rights reserved.
@@ -68,7 +68,7 @@ extern "C" {
 using namespace std;
 using namespace Dwm;
 
-static const Dwm::SvnTag svntag("@(#) $DwmPath: dwm/mcplex/mcblock/tags/mcblock-0.1.1/apps/mcblockd/DwmMcBlockTcpDrop.cc 9420 $");
+static const Dwm::SvnTag svntag("@(#) $DwmPath: dwm/mcplex/mcblock/tags/mcblock-0.3.3/apps/mcblockd/DwmMcBlockTcpDrop.cc 12157 $");
 
 #define TCPDROP_FOREIGN     0
 #define TCPDROP_LOCAL       1
@@ -168,15 +168,49 @@ namespace Dwm {
     //------------------------------------------------------------------------
     bool TcpDropPrefix(Ipv4Prefix & prefix)
     {
-#ifdef __APPLE__
+#if ! defined(__FreeBSD__)
       return false;
-#else
+#elif (__FreeBSD__ >= 12)
+      bool  rc = true;
+      struct xinpgen *head, *xinp;
+      struct xtcpcb  *xtp;
+      struct xinpcb  *xip;
 
+#define XINP_NEXT(xinp)                                         \
+      ((struct xinpgen *)((uintptr_t)(xinp) + (xinp)->xig_len))
+
+      head = GetXPcbList("net.inet.tcp.pcblist");
+      for (xinp = XINP_NEXT(head); xinp->xig_len > sizeof *xinp;
+           xinp = XINP_NEXT(xinp)) {
+        xtp = (struct xtcpcb *)xinp;
+        xip = &xtp->xt_inp;
+
+        // XXX Check protocol?
+        
+        // Ignore PCBs which were freed during copyout.
+        if (xip->inp_gencnt > head->xig_gen)
+          continue;
+        // Skip listening sockets.
+        if (xtp->t_state == TCPS_LISTEN)
+          continue;
+        //  Skip IPv6.
+        if ((xip->inp_inc.inc_flags & INC_ISIPV6) != 0)
+          continue;
+        
+        if (prefix.Contains(xip->inp_inc.inc_faddr.s_addr)) {
+          if (! TcpDropIpv4Conn(&xip->inp_inc)) {
+            rc = false;
+          }
+        }
+      }
+      free(head);
+      return rc;
+#elif (__FreeBSD__ == 11)
       bool  rc = true;
       struct xinpgen *head, *xinp;
       struct xtcpcb  *xpcb;
-      struct tcpcb   *tp;
-      struct inpcb   *inp;
+      struct tcpcb  *tp;
+      struct inpcb  *inp;
 
 #define XINP_NEXT(xinp)                                         \
       ((struct xinpgen *)((uintptr_t)(xinp) + (xinp)->xig_len))
