@@ -1,6 +1,6 @@
 //===========================================================================
 // @(#) $Name:$
-// @(#) $Id: mcblockd.cc 9274 2017-04-26 04:52:50Z dwm $
+// @(#) $Id: mcblockd.cc 12099 2022-12-04 01:14:17Z dwm $
 //===========================================================================
 //  Copyright (c) Daniel W. McRobb 2017
 //  All rights reserved.
@@ -44,7 +44,8 @@ extern "C" {
   #include <unistd.h>
 }
 
-#include "DwmAuthPKCrypto.hh"
+#include <atomic>
+
 #include "DwmDaemonUtils.hh"
 #include "DwmOptArgs.hh"
 #include "DwmSignal.hh"
@@ -53,11 +54,13 @@ extern "C" {
 #include "DwmCfg2Json.hh"
 #include "DwmMcBlockServer.hh"
 
-static const Dwm::SvnTag  svntag("@(#) $DwmPath: dwm/mcplex/mcblock/tags/mcblock-0.1.1/apps/mcblockd/mcblockd.cc 9274 $");
+static const Dwm::SvnTag  svntag("@(#) $DwmPath: dwm/mcplex/mcblock/tags/mcblock-0.3.3/apps/mcblockd/mcblockd.cc 12099 $");
 
 using namespace std;
 using namespace Dwm;
 
+static atomic<bool>  g_sigTermReceived = false;
+
 //----------------------------------------------------------------------------
 //!  
 //----------------------------------------------------------------------------
@@ -73,30 +76,42 @@ static void SavePID(const string & pidFile)
   return;
 }
 
+//----------------------------------------------------------------------------
+//!  
+//----------------------------------------------------------------------------
+static void SigTermHandler(int sigNum)
+{
+  if (SIGTERM == sigNum) {
+    Syslog(LOG_INFO, "SIGTERM received.");
+    g_sigTermReceived = true;
+  }
+  return;
+}
+
 //----------------------------------------------------------------------------
 //!  
 //----------------------------------------------------------------------------
 static void ConfigureSyslog(const Cfg2Json & mcblockConfig, int logOpts)
 {
-  const Json::Value *jvp = mcblockConfig.Find("Syslog");
+  const nlohmann::json  *jvp = mcblockConfig.Find("Syslog");
   if (jvp) {
-    const Json::Value  *p = mcblockConfig.Find(jvp, "Facility");
-    if (p && p->isString()) {
-      SysLogger::Open("mcblockd", logOpts, p->asString());
+    const nlohmann::json  *p = mcblockConfig.Find(jvp, "Facility");
+    if (p && p->is_string()) {
+      SysLogger::Open("mcblockd", logOpts, p->get<string>());
     }
     else {
       SysLogger::Open("mcblockd", logOpts, "daemon");
     }
     p = mcblockConfig.Find(jvp, "MinimumPriority");
-    if (p && p->isString()) {
-      SysLogger::MinimumPriority(p->asString());
+    if (p && p->is_string()) {
+      SysLogger::MinimumPriority(p->get<string>());
     }
     else {
       SysLogger::MinimumPriority("info");
     }
     p = mcblockConfig.Find(jvp, "ShowFileLocations");
-    if (p && p->isString()) {
-      SysLogger::ShowFileLocation(p->asString() == "true");
+    if (p && p->is_string()) {
+      SysLogger::ShowFileLocation(p->get<string>() == "true");
     }
     else {
       SysLogger::ShowFileLocation(false);
@@ -131,7 +146,7 @@ int main(int argc, char *argv[])
   SigPipe.Block();
 
   Signal SigTerm(SIGTERM);
-  SigTerm.Block();
+  SigTerm.PushHandler(SigTermHandler);
 
   string  pidFile = optargs.Get<string>('p');
   if (pidFile.empty()) {
@@ -152,13 +167,16 @@ int main(int argc, char *argv[])
     if (server.Start()) {
       for (;;) {
         sleep(1);
-        if (SigTerm.IsPending()) {
+        if (g_sigTermReceived) {
           break;
         }
       }
       server.Stop();
       rc = 0;
     }
+    else {
+      Syslog(LOG_ERR, "Failed to start server!");
+    }
   }
   else {
     Syslog(LOG_ERR, "Failed to parse config file!");
