@@ -1,6 +1,6 @@
 //===========================================================================
 // @(#) $Name:$
-// @(#) $Id: mcblocklog.cc 9420 2017-06-04 05:28:54Z dwm $
+// @(#) $Id: mcblocklog.cc 12114 2022-12-05 04:02:55Z dwm $
 //===========================================================================
 //  Copyright (c) Daniel W. McRobb 2017
 //  All rights reserved.
@@ -54,16 +54,13 @@ extern "C" {
 #include "DwmIpv4Prefix.hh"
 #include "DwmOptArgs.hh"
 #include "DwmSignal.hh"
-#include "DwmSocket.hh"
 #include "DwmSvnTag.hh"
 #include "DwmSysLogger.hh"
-#include "DwmAuthPeerAuthenticator.hh"
-#include "DwmAuth.hh"
 #include "DwmMcBlockLogRegexMatcher.hh"
 #include "DwmMcBlockRequestMessage.hh"
 #include "DwmMcBlockResponseMessage.hh"
 
-static const Dwm::SvnTag svntag("@(#) $DwmPath: dwm/mcplex/mcblock/tags/mcblock-0.1.1/apps/mcblocklog/mcblocklog.cc 9420 $");
+static const Dwm::SvnTag svntag("@(#) $DwmPath: dwm/mcplex/mcblock/tags/mcblock-0.3.3/apps/mcblocklog/mcblocklog.cc 12114 $");
 
 using namespace std;
 using namespace Dwm;
@@ -71,7 +68,7 @@ using namespace Dwm;
 //----------------------------------------------------------------------------
 //!  
 //----------------------------------------------------------------------------
-static bool ConnectToMcBlock(const string & host, Socket & s)
+static bool ConnectToMcBlock(const string & host, Credence::Peer & peer)
 {
   bool  rc = false;
   struct hostent  *hostEntry = gethostbyname(host.c_str());
@@ -79,52 +76,25 @@ static bool ConnectToMcBlock(const string & host, Socket & s)
     Syslog(LOG_ERR, "Host '%s' not found", host.c_str());
     exit(1);
   }
-  
-  if (s.Open(PF_INET, SOCK_STREAM, 0)) {
-    struct sockaddr_in  servAddr;
-    memset(&servAddr, 0, sizeof(servAddr));
-    servAddr.sin_len = sizeof(servAddr);
-    servAddr.sin_family = AF_INET;
-    servAddr.sin_port = htons(1001);
-    servAddr.sin_addr.s_addr = *(in_addr_t *)hostEntry->h_addr_list[0];
-    if (s.Connect(servAddr)) {
-      int  noDelay = 1;
-      s.Setsockopt(IPPROTO_TCP, TCP_NODELAY, &noDelay, sizeof(noDelay));
-      rc = true;
-    }
-    else {
-      Syslog(LOG_ERR, "Failed to connect to '%s'", host.c_str());
-    }
-  }
-  return rc;
-}
-
-//----------------------------------------------------------------------------
-//!  
-//----------------------------------------------------------------------------
-static bool Authenticate(Dwm::Socket & s, string & theirId,
-                         string & agreedKey)
-{
-  bool  rc = false;
-  Auth::PeerAuthenticator
-    peerAuth("/usr/local/etc/mcblocklog/id_rsa",
-             "/usr/local/etc/mcblocklog/known_services");
-  if (peerAuth.Authenticate(s, theirId, agreedKey)) {
+  if (peer.Connect(host, 1001)) {
     rc = true;
   }
+  else {
+    Syslog(LOG_ERR, "Failed to connect to '%s'", host.c_str());
+  }
   return rc;
 }
 
 //----------------------------------------------------------------------------
 //!  
 //----------------------------------------------------------------------------
-bool RequestResponse(Socket & s, const string & secret,
+bool RequestResponse(Credence::Peer & peer,
                      const McBlock::RequestMessage & request,
                      McBlock::ResponseMessage & response)
 {
   bool  rc = false;
-  if (request.Write(s, secret)) {
-    if (response.Read(s, secret)) {
+  if (request.Write(peer)) {
+    if (response.Read(peer)) {
       rc = true;
     }
     else {
@@ -143,16 +113,17 @@ bool RequestResponse(Socket & s, const string & secret,
 static void LogHit(const string & mcblockServer, const string & tableName,
                    const Ipv4Address & addr)
 {
-  Socket  sock;
+  Credence::Peer peer;
   
-  if (ConnectToMcBlock(mcblockServer, sock)) {
-    string  theirId, sharedKey;
-    if (Authenticate(sock, theirId, sharedKey)) {
+  if (ConnectToMcBlock(mcblockServer, peer)) {
+    Credence::KeyStash   keyStash("/usr/local/etc/mcblocklog");
+    Credence::KnownKeys  knownKeys("/usr/local/etc/mcblocklog");
+    if (peer.Authenticate(keyStash, knownKeys)) {
       Dwm::TimeValue64  now(true);
       McBlock::ResponseMessage  resp;
       McBlock::RequestMessage   req((McBlock::LogHitReq(tableName, addr,
                                                         now)));
-      RequestResponse(sock, sharedKey, req, resp);
+      RequestResponse(peer, req, resp);
     }
   }
   return;
