@@ -1,6 +1,6 @@
 //===========================================================================
 // @(#) $Name:$
-// @(#) $Id: DwmMcBlockRequestMessage.cc 9686 2017-06-30 08:10:06Z dwm $
+// @(#) $Id: DwmMcBlockRequestMessage.cc 12091 2022-11-29 05:05:27Z dwm $
 //===========================================================================
 //  Copyright (c) Daniel W. McRobb 2017
 //  All rights reserved.
@@ -40,10 +40,10 @@
 //---------------------------------------------------------------------------
 
 #include "DwmSvnTag.hh"
-#include "DwmAuthSymCryptoMessage.hh"
+#include "DwmCredencePeer.hh"
 #include "DwmMcBlockRequestMessage.hh"
 
-static const Dwm::SvnTag svntag("@(#) $DwmPath: dwm/mcplex/mcblock/tags/mcblock-0.1.1/classes/src/DwmMcBlockRequestMessage.cc 9686 $");
+static const Dwm::SvnTag svntag("@(#) $DwmPath: dwm/mcplex/mcblock/tags/mcblock-0.3.3/classes/src/DwmMcBlockRequestMessage.cc 12091 $");
 
 using namespace std;
 
@@ -149,14 +149,14 @@ namespace Dwm {
     //------------------------------------------------------------------------
     //!  
     //------------------------------------------------------------------------
-    RequestMessage::RequestMessage(const Json::Value & req)
+    RequestMessage::RequestMessage(const nlohmann::json & req)
         : _value(req)
     {}
 
     //------------------------------------------------------------------------
     //!  
     //------------------------------------------------------------------------
-    RequestMessage::RequestMessage(const Json::Value && req)
+    RequestMessage::RequestMessage(const nlohmann::json && req)
         : _value(std::move(req))
     {
     }
@@ -164,16 +164,14 @@ namespace Dwm {
     //------------------------------------------------------------------------
     //!  
     //------------------------------------------------------------------------
-    bool RequestMessage::Read(int fd, const string & secret)
+    bool RequestMessage::Read(Credence::Peer & peer)
     {
       bool  rc = false;
       _value.clear();
-      if (fd >= 0) {
-        Auth::SymCrypto::Message  msg(secret);
-        if (msg.Read(fd) > 0) {
-          Json::Reader  jsonReader;
-          rc = jsonReader.parse(msg.Value(), _value);
-        }
+      std::string  s;
+      if (peer.Receive(s)) {
+        _value = nlohmann::json::parse(s, nullptr, false);
+        rc = (! _value.is_discarded());
       }
       return rc;
     }
@@ -181,23 +179,18 @@ namespace Dwm {
     //------------------------------------------------------------------------
     //!  
     //------------------------------------------------------------------------
-    bool RequestMessage::Write(int fd, const string & secret) const
+    bool RequestMessage::Write(Credence::Peer & peer) const
     {
       bool  rc = false;
-      if (fd >= 0) {
-        Json::FastWriter  fw;
-        Auth::SymCrypto::Message  msg(secret, fw.write(_value));
-        if (msg.Write(fd) == msg.StreamedLength()) {
-          rc = true;
-        }
-      }
+      std::string  s = _value.dump();
+      rc = peer.Send(s);
       return rc;
     }
     
     //------------------------------------------------------------------------
     //!  
     //------------------------------------------------------------------------
-    const Json::Value & RequestMessage::Json() const
+    const nlohmann::json & RequestMessage::Json() const
     {
       return _value;
     }
