@@ -1,6 +1,6 @@
 //===========================================================================
 // @(#) $Name:$
-// @(#) $Id: DwmMcBlockResponseMessage.cc 9242 2017-04-25 17:27:24Z dwm $
+// @(#) $Id: DwmMcBlockResponseMessage.cc 12091 2022-11-29 05:05:27Z dwm $
 //===========================================================================
 //  Copyright (c) Daniel W. McRobb 2017
 //  All rights reserved.
@@ -41,10 +41,9 @@
 
 #include "DwmSvnTag.hh"
 #include "DwmSysLogger.hh"
-#include "DwmAuthSymCryptoMessage.hh"
 #include "DwmMcBlockResponseMessage.hh"
 
-static const Dwm::SvnTag svntag("@(#) $DwmPath: dwm/mcplex/mcblock/tags/mcblock-0.1.1/classes/src/DwmMcBlockResponseMessage.cc 9242 $");
+static const Dwm::SvnTag svntag("@(#) $DwmPath: dwm/mcplex/mcblock/tags/mcblock-0.3.3/classes/src/DwmMcBlockResponseMessage.cc 12091 $");
 
 using namespace std;
 
@@ -62,21 +61,21 @@ namespace Dwm {
     //------------------------------------------------------------------------
     //!  
     //------------------------------------------------------------------------
-    ResponseMessage::ResponseMessage(const Json::Value & value)
+    ResponseMessage::ResponseMessage(const nlohmann::json & value)
         : _value(value)
     {}
 
     //------------------------------------------------------------------------
     //!  
     //------------------------------------------------------------------------
-    ResponseMessage::ResponseMessage(const Json::Value && value)
+    ResponseMessage::ResponseMessage(nlohmann::json && value)
         : _value(std::move(value))
     {}
     
     //------------------------------------------------------------------------
     //!  
     //------------------------------------------------------------------------
-    const Json::Value & ResponseMessage::Json() const
+    const nlohmann::json & ResponseMessage::Json() const
     {
       return _value;
     }
@@ -84,7 +83,7 @@ namespace Dwm {
     //------------------------------------------------------------------------
     //!  
     //------------------------------------------------------------------------
-    const Json::Value & ResponseMessage::Json(const Json::Value & value)
+    const nlohmann::json & ResponseMessage::Json(const nlohmann::json & value)
     {
       _value = value;
       return _value;
@@ -93,44 +92,39 @@ namespace Dwm {
     //------------------------------------------------------------------------
     //!  
     //------------------------------------------------------------------------
-    bool ResponseMessage::Read(int fd, const string & secret)
+    bool ResponseMessage::Read(Credence::Peer & peer)
     {
       bool  rc = false;
       _value.clear();
       _value = "";
-      if (fd >= 0) {
-        Auth::SymCrypto::Message  msg(secret);
-        if (msg.Read(fd) > 0) {
-          Json::Reader  jsonReader;
-          if (jsonReader.parse(msg.Value(), _value)) {
-            rc = true;
-          }
-          else {
-            Syslog(LOG_ERR, "Failed to parse response message");
-          }
+      std::string  s;
+      if (peer.Receive(s)) {
+        _value = nlohmann::json::parse(s, nullptr, false);
+        if (! _value.is_discarded()) {
+          rc = true;
         }
         else {
-          Syslog(LOG_ERR, "Failed to read response message");
+          Syslog(LOG_ERR, "Failed to parse response message");
         }
       }
+      else {
+        Syslog(LOG_ERR, "Failed to read response message");
+      }
       return rc;
     }
     
     //------------------------------------------------------------------------
     //!  
     //------------------------------------------------------------------------
-    bool ResponseMessage::Write(int fd, const string & secret) const
+    bool ResponseMessage::Write(Credence::Peer & peer) const
     {
       bool  rc = false;
-      if (fd >= 0) {
-        Json::FastWriter  fw;
-        Auth::SymCrypto::Message  msg(secret, fw.write(_value));
-        if (msg.Write(fd) == msg.StreamedLength()) {
-          rc = true;
-        }
-        else {
-          Syslog(LOG_ERR, "Failed to write response message");
-        }
+      std::string  s = _value.dump();
+      if (peer.Send(s)) {
+        rc = true;
+      }
+      else {
+        Syslog(LOG_ERR, "Failed to write response message");
       }
       return rc;
     }
