@@ -1,6 +1,6 @@
 //===========================================================================
 // @(#) $Name:$
-// @(#) $Id: DwmMcBlockServerConfig.cc 9275 2017-04-26 04:54:13Z dwm $
+// @(#) $Id: DwmMcBlockServerConfig.cc 12091 2022-11-29 05:05:27Z dwm $
 //===========================================================================
 //  Copyright (c) Daniel W. McRobb 2017
 //  All rights reserved.
@@ -42,7 +42,7 @@
 #include "DwmSvnTag.hh"
 #include "DwmMcBlockServerConfig.hh"
 
-static const Dwm::SvnTag svntag("@(#) $DwmPath: dwm/mcplex/mcblock/tags/mcblock-0.1.1/classes/src/DwmMcBlockServerConfig.cc 9275 $");
+static const Dwm::SvnTag svntag("@(#) $DwmPath: dwm/mcplex/mcblock/tags/mcblock-0.3.3/classes/src/DwmMcBlockServerConfig.cc 12091 $");
 
 using namespace std;
 
@@ -56,57 +56,42 @@ namespace Dwm {
     ServerConfig::ServerConfig(const Cfg2Json & config)
         : _pfdevpath("/dev/pf"), _tcpAddr(INADDR_ANY), _tcpPort(1001),
           _mcastAddr("224.0.0.42"), _mcastPort(1001),
-          _privateKeyFile("/usr/local/etc/mcblockd/id_rsa"),
-          _authorizedKeysFile("/usr/local/etc/mcblockd/authorized_keys"),
-          _dwmRDAPPrivateKeyFile("/usr/local/etc/mcblockd/id_rsa"),
-          _knownServicesFile("/usr/local/etc/mcblockd/known_services"),
+          _keyDirectory("/usr/local/etc/mcblockd"),
           _dwmRDAPServer("127.0.0.1"), _databaseDirectory("/etc")
     {
-      const Json::Value *jvp = config.Find("Server");
+      const nlohmann::json  *jvp = config.Find("Server");
       if (jvp) {
-        const Json::Value *p = config.Find(jvp, "PF/Device");
-        if (p && p->isString()) {
-          _pfdevpath = p->asString();
+        const nlohmann::json  *p = config.Find(jvp, "PF/Device");
+        if (p && p->is_string()) {
+          _pfdevpath = p->get<string>();
         }
         p = config.Find(jvp, "TCP/Address");
-        if (p && p->isString()) {
-          _tcpAddr = p->asString();
+        if (p && p->is_string()) {
+          _tcpAddr = p->get<string>();
         }
         p = config.Find(jvp, "TCP/Port");
-        if (p && p->isString()) {
-          _tcpPort = stoul(p->asString());
+        if (p && p->is_string()) {
+          _tcpPort = stoul(p->get<string>());
         }
         p = config.Find("Multicast/Address");
-        if (p && p->isString()) {
-          _mcastAddr = p->asString();
+        if (p && p->is_string()) {
+          _mcastAddr = p->get<string>();
         }
         p = config.Find(jvp, "Multicast/Port");
-        if (p && p->isString()) {
-          _mcastPort = stoul(p->asString());
+        if (p && p->is_string()) {
+          _mcastPort = stoul(p->get<string>());
         }
-        p = config.Find(jvp, "Auth/PrivKeyFile");
-        if (p && p->isString()) {
-          _privateKeyFile = p->asString();
-        }
-        p = config.Find(jvp, "Auth/AuthorizedKeysFile");
-        if (p && p->isString()) {
-          _authorizedKeysFile = p->asString();
-        }
-        p = config.Find(jvp, "DwmRDAP/PrivKeyFile");
-        if (p && p->isString()) {
-          _dwmRDAPPrivateKeyFile = p->asString();
-        }
-        p = config.Find(jvp, "DwmRDAP/KnownServices");
-        if (p && p->isString()) {
-          _knownServicesFile = p->asString();
+        p = config.Find(jvp, "Auth/KeyDirectory");
+        if (p && p->is_string()) {
+          _keyDirectory = p->get<string>();
         }
         p = config.Find(jvp, "DwmRDAP/Server");
-        if (p && p->isString()) {
-          _dwmRDAPServer = p->asString();
+        if (p && p->is_string()) {
+          _dwmRDAPServer = p->get<string>();
         }
         p = config.Find(jvp, "DatabaseDirectory");
-        if (p && p->isString()) {
-          _databaseDirectory = p->asString();
+        if (p && p->is_string()) {
+          _databaseDirectory = p->get<string>();
         }
       }
     }
@@ -200,33 +185,9 @@ namespace Dwm {
     //------------------------------------------------------------------------
     //!  
     //------------------------------------------------------------------------
-    const string & ServerConfig::PrivateKeyFile() const
+    const string & ServerConfig::KeyDirectory() const
     {
-      return _privateKeyFile;
-    }
-
-    //------------------------------------------------------------------------
-    //!  
-    //------------------------------------------------------------------------
-    const string & ServerConfig::AuthorizedKeysFile() const
-    {
-      return _authorizedKeysFile;
-    }
-    
-    //------------------------------------------------------------------------
-    //!  
-    //------------------------------------------------------------------------
-    const string & ServerConfig::DwmRDAPPrivateKeyFile() const
-    {
-      return _dwmRDAPPrivateKeyFile;
-    }
-    
-    //------------------------------------------------------------------------
-    //!  
-    //------------------------------------------------------------------------
-    const string & ServerConfig::KnownServicesFile() const
-    {
-      return _knownServicesFile;
+      return _keyDirectory;
     }
     
     //------------------------------------------------------------------------
@@ -248,16 +209,14 @@ namespace Dwm {
     //------------------------------------------------------------------------
     //!  
     //------------------------------------------------------------------------
-    Json::Value ServerConfig::Json() const
+    nlohmann::json ServerConfig::Json() const
     {
-      Json::Value  value;
+      nlohmann::json  value;
       value["pfdevpath"] = _pfdevpath;
       value["tcpAddr"] = (string)_tcpAddr;
       value["tcpPort"] = _tcpPort;
       value["mcastAddr"] = (string)_mcastAddr;
       value["mcastPort"] = _mcastPort;
-      value["dwmRDAPPrivateKeyFile"] = _dwmRDAPPrivateKeyFile;
-      value["knownServicesFile"] = _knownServicesFile;
       value["dwmRDAPServer"] = _dwmRDAPServer;
       value["databaseDirectory"] = _databaseDirectory;
       return value;
@@ -266,16 +225,14 @@ namespace Dwm {
     //------------------------------------------------------------------------
     //!  
     //------------------------------------------------------------------------
-    Json::Value ServerConfig::Json(const Json::Value & value)
+    nlohmann::json ServerConfig::Json(const nlohmann::json & value)
     {
-      _pfdevpath = value["pfdevpath"].asString();
-      _tcpAddr = Ipv4Address(value["tcpAddr"].asString());
-      _tcpPort = value["tcpPort"].asUInt();
-      _mcastAddr = Ipv4Address(value["mcastAddr"].asString());
-      _mcastPort = value["mcastPort"].asUInt();
-      _dwmRDAPPrivateKeyFile = value["dwmRDAPPrivateKeyFile"].asString();
-      _knownServicesFile = value["knownServicesFile"].asString();
-      _dwmRDAPServer = value["dwmRDAPServer"].asString();
+      _pfdevpath = value["pfdevpath"].get<string>();
+      _tcpAddr = Ipv4Address(value["tcpAddr"].get<string>());
+      _tcpPort = value["tcpPort"].get<uint16_t>();
+      _mcastAddr = Ipv4Address(value["mcastAddr"].get<string>());
+      _mcastPort = value["mcastPort"].get<uint16_t>();
+      _dwmRDAPServer = value["dwmRDAPServer"].get<string>();
       return Json();
     }
     
@@ -289,8 +246,7 @@ namespace Dwm {
               && (_tcpPort == config._tcpPort)
               && (_mcastAddr == config._mcastAddr)
               && (_mcastPort == config._mcastPort)
-              && (_dwmRDAPPrivateKeyFile == config._dwmRDAPPrivateKeyFile)
-              && (_knownServicesFile == config._knownServicesFile)
+              && (_keyDirectory == config._keyDirectory)
               && (_dwmRDAPServer == config._dwmRDAPServer)
               && (_databaseDirectory == config._databaseDirectory));
     }
