@@ -1,6 +1,6 @@
 //===========================================================================
 // @(#) $Name:$
-// @(#) $Id: TestDwmMcBlockRequestMessage.cc 8841 2017-03-22 08:34:07Z dwm $
+// @(#) $Id: TestDwmMcBlockRequestMessage.cc 12110 2022-12-05 01:39:07Z dwm $
 //===========================================================================
 //  Copyright (c) Daniel W. McRobb 2017
 //  All rights reserved.
@@ -49,7 +49,7 @@ extern "C" {
 #include "DwmUnitAssert.hh"
 #include "DwmMcBlockRequestMessage.hh"
 
-static const Dwm::SvnTag svntag("@(#) $DwmPath: dwm/mcplex/mcblock/tags/mcblock-0.1.1/classes/tests/TestDwmMcBlockRequestMessage.cc 8841 $");
+static const Dwm::SvnTag svntag("@(#) $DwmPath: dwm/mcplex/mcblock/tags/mcblock-0.3.3/classes/tests/TestDwmMcBlockRequestMessage.cc 12110 $");
 
 using namespace std;
 using namespace Dwm;
@@ -57,49 +57,6 @@ using McBlock::RequestMessage;
 
 bool  g_printJson = false;
 
-//----------------------------------------------------------------------------
-//!  
-//----------------------------------------------------------------------------
-static void TestDescriptorIO()
-{
-  vector<Ipv4Prefix>  activatePrefixes = {
-    Ipv4Prefix("10.21.22/24"),
-    Ipv4Prefix("10.1.1/24")
-  };
-  
-  vector<RequestMessage>  messages = {
-    RequestMessage(McBlock::ActivateReq("foo", activatePrefixes)),
-    RequestMessage(McBlock::DeactivateReq("foo", Ipv4Prefix("10.22.23/24"))),
-    RequestMessage(McBlock::GetActiveReq("foo")),
-    RequestMessage(McBlock::GetMatchesReq("foo", Ipv4Prefix("10.23.24.25/32")))
-  };
-  int  fd = open("./TestDwmMcBlockRequestMessage.tmp",
-                 O_WRONLY|O_CREAT|O_TRUNC, 0644);
-  if (UnitAssert(fd >= 0)) {
-    for (auto msg : messages) {
-      UnitAssert(msg.Write(fd, "MySeCrEt!"));
-    }
-    close(fd);
-  }
-
-  fd = open("./TestDwmMcBlockRequestMessage.tmp", O_RDONLY);
-  if (UnitAssert(fd >= 0)) {
-    for (auto msg : messages) {
-      RequestMessage  rmsg;
-      UnitAssert(rmsg.Read(fd, "MySeCrEt!"));
-      UnitAssert(rmsg.Json() == msg.Json());
-      if (g_printJson) {
-        Json::FastWriter  fw;
-        cout << fw.write(rmsg.Json());
-      }
-    }
-    close(fd);
-  }
-  remove("./TestDwmMcBlockRequestMessage.tmp");
-  
-  return;
-}
-
 //----------------------------------------------------------------------------
 //!  
 //----------------------------------------------------------------------------
@@ -112,7 +69,6 @@ int main(int argc, char *argv[])
   optargs.Parse(argc, argv);
   g_printJson = optargs.Get<bool>('p');
   
-  TestDescriptorIO();
 
   if (Assertions::Total().Failed()) {
     Assertions::Print(std::cerr, true);
