@@ -1,6 +1,6 @@
 //===========================================================================
 // @(#) $Name:$
-// @(#) $Id: TestDwmMcBlockResponseMessage.cc 8832 2017-03-21 17:16:03Z dwm $
+// @(#) $Id: TestDwmMcBlockResponseMessage.cc 12110 2022-12-05 01:39:07Z dwm $
 //===========================================================================
 //  Copyright (c) Daniel W. McRobb 2017
 //  All rights reserved.
@@ -49,7 +49,7 @@ extern "C" {
 #include "DwmUnitAssert.hh"
 #include "DwmMcBlockResponseMessage.hh"
 
-static const Dwm::SvnTag svntag("@(#) $DwmPath: dwm/mcplex/mcblock/tags/mcblock-0.1.1/classes/tests/TestDwmMcBlockResponseMessage.cc 8832 $");
+static const Dwm::SvnTag svntag("@(#) $DwmPath: dwm/mcplex/mcblock/tags/mcblock-0.3.3/classes/tests/TestDwmMcBlockResponseMessage.cc 12110 $");
 
 using namespace std;
 using namespace Dwm;
@@ -62,12 +62,12 @@ static bool  g_printJson;
 //----------------------------------------------------------------------------
 static void TestDescriptorIO()
 {
-  Json::Value  prefixes;
+  nlohmann::json  prefixes;
   prefixes[0] = "10.11.12/24";
   prefixes[1] = "10.12.13/24";
   prefixes[2] = "10.13.14/24";
 
-  Json::Value  val1, val2, val3, val4;
+  nlohmann::json  val1, val2, val3, val4;
   val1["prefix"] = "10.11.12/24";
   val2["prefix"] = "10.12.13/24";
   val3["prefixes"] = prefixes;
@@ -80,55 +80,12 @@ static void TestDescriptorIO()
     ResponseMessage(val4)
   };
 
-  UnitAssert(messages[2].Json()["prefixes"].isArray());
+  UnitAssert(messages[2].Json()["prefixes"].is_array());
   for (int i = 0; i < prefixes.size(); ++i) {
-    UnitAssert(messages[2].Json()["prefixes"][i].isString());
-    UnitAssert(messages[2].Json()["prefixes"][i].asString()
-               == prefixes[i].asString());
+    UnitAssert(messages[2].Json()["prefixes"][i].is_string());
+    UnitAssert(messages[2].Json()["prefixes"][i].get<string>()
+               == prefixes[i].get<string>());
   }
-  
-  int  fd = open("./TestDwmMcBlockResponseMessage.tmp",
-                 O_WRONLY|O_CREAT|O_TRUNC, 0644);
-  if (UnitAssert(fd >= 0)) {
-    for (auto msg : messages) {
-      UnitAssert(msg.Write(fd, "MySeCrEt!"));
-    }
-    close(fd);
-  }
-  
-  fd = open("./TestDwmMcBlockResponseMessage.tmp", O_RDONLY);
-  if (UnitAssert(fd >= 0)) {
-    vector<ResponseMessage>  rmsgs;
-    ResponseMessage  msg;
-    while (msg.Read(fd, "MySeCrEt!")) {
-      rmsgs.push_back(msg);
-    }
-    close(fd);
-    if (UnitAssert(rmsgs.size() == messages.size())) {
-      for (int i = 0; i < messages.size(); ++i) {
-        UnitAssert(rmsgs[i].Json() == messages[i].Json());
-        if (i == 2) {
-          if (UnitAssert(rmsgs[i].Json()["prefixes"].isArray())) {
-            if (UnitAssert(rmsgs[i].Json()["prefixes"].size()
-                           == prefixes.size())) {
-              for (int p = 0; p < rmsgs[i].Json()["prefixes"].size(); ++p) {
-                if (UnitAssert(rmsgs[i].Json()["prefixes"][p].isString())) {
-                  UnitAssert(rmsgs[i].Json()["prefixes"][p].asString()
-                             == prefixes[p].asString());
-                }
-              }
-            }
-          }
-        }
-        if (g_printJson) {
-          Json::FastWriter  fw;
-          cout << fw.write(rmsgs[i].Json());
-        }
-      }
-    }
-  }
-  remove("./TestDwmMcBlockResponseMessage.tmp");
-  
   return;
 }
 
